#!/usr/bin/env python
# -*- mode: landisk-python; coding: utf-8; -*-
# vim:ts=4 sw=4 sts=4 ai si et sta

"""FAL command for a cloud service."""

import urlparse
import libfal.falcmd as falcmd
import pal_info
from iobase.xmldata import (XmlConfNode, ANY_CHILD_NAME,)
from iobase.exception import (NotFoundError,)
from ionas.request import (Entry,)
from libfal.util import (get_proxy, NoProxy, servicectrl_suspend,
                            servicectrl_resume)
from fal_info import (ReqTypes, Results,)
from fal_config import (FUNC_TO_SERVICE_MAP,)

from pkgaddon.landisk_cloud_common_config import (CLOUD_CONF_FILE,)


CLOUD_NAME = "cloud"
ID_NAME = "id"
ID_MIN = 0
NAME_NAME = "name"
OLDNAME_XML_TMP = "<oldname>%s</oldname>"
ANY_LIST_ITEM_XPATH = ANY_CHILD_NAME + "/" + ANY_CHILD_NAME
GET_ANY_ID_XPATH = ANY_LIST_ITEM_XPATH + "/id"
FIND_BY_ID_XPATH_TMPL = ANY_LIST_ITEM_XPATH + '/id[text()="%s"]/..'

FLETSAZUKERU_NAME = "fletsazukeru"
AREA_NAME = "area"
HTTPS_PROTOCOL = "https"

TO_LIST_TAG_NANE = {
        "dropbox": "dropboxlist",
        "amazons3": "amazons3list",
        "cloudn": "cloudnlist",
        "nifcloud": "nifcloudlist",
        "azure": "azurelist",
        "box": "boxlist",
        "onedrive": "onedrivelist",
        "onedrivebusiness": "onedrivebusinesslist",
        "googledrive": "googledrivelist",
        "fletsazukeru": "fletsazukerulist",
        "mdcloud": "mdcloudlist",
    }


NASDSYNC2_SERVICE = "nasdsync2"
# Add the service entry for the clone package
FUNC_TO_SERVICE_MAP[NASDSYNC2_SERVICE] = NASDSYNC2_SERVICE


class FalCloud(falcmd.FalCmd):
    """FAL Cloud command implementation class."""

    def __get_proxy(self, url):
        """Return proxy for "url"."""

        try:
            return get_proxy(url)
        except NoProxy:
            pass

        return ""

    def __get_proxy_for_host_name(self, host_name):
        """Return proxy for server corresponding a "host_name"."""

        target_url = urlparse.urlunparse((HTTPS_PROTOCOL,
                                host_name, "", "", "", ""))

        return self.__get_proxy(target_url)

    def __try_append_proxy_tag(self, target_node):
        """Try append proxy info to "target_node"."""

        proxy_info = self.__get_proxy_for_host_name("www.iodata.jp")

        if proxy_info:
            target_node.append_child(proxy_info)

    def __load_config(self, file_name):
        """Load a 'file_name' file setting and return XML data ."""

        return self.send_entry_with_file(Entry(ReqTypes.LOAD),
                                        pal_info.PAL_CONFFILE_PATH,
                                        file_name).get()

    def __save_config(self, file_name, xml_data):
        """Save the "xml_data" XML data to a 'file_name' file."""

        entry = Entry(ReqTypes.SAVE)
        entry.set(xml_data)
        return self.send_entry_with_file(entry,
                                pal_info.PAL_CONFFILE_PATH, file_name)

    def __check_handler(self, entry):
        """Handle check request and set entry to result."""

        checked_node = entry.get().get_node(ANY_CHILD_NAME)
        cloud_conf = self.__load_config(CLOUD_CONF_FILE)
        try:
            target_id = checked_node.get_node(ID_NAME).get_text()
        except:
            self.exception("target id is not specified!!")
            entry.set_result(Results.CHECK_ERROR)
            return entry

        try:
            target_node = cloud_conf.get_node(FIND_BY_ID_XPATH_TMPL %
                            (target_id,))
        except:
            self.exception("target %d is not found" % (target_id,))
            entry.set_result(Results.CHECK_ERROR)
            return entry

        check_entry = Entry(ReqTypes.CHECK, CLOUD_NAME)
        check_entry.get().append_child(target_node)

        self.__try_append_proxy_tag(check_entry.get())

        result_entry = self.send_entry(check_entry, pal_info.PAL_CLOUD_PATH)
        entry.set_result(result_entry.get_result())
        return entry

    def __add_handler(self, entry):
        """Handle add request and set entry to result."""

        added_node = entry.get().get_node(ANY_CHILD_NAME)
        try:
            # Force delete ID tag.
            added_node.delete_children(ID_NAME)
        except:
            # Ignore error.
            pass

        cloud_conf = self.__load_config(CLOUD_CONF_FILE)
        target_name = added_node.get_node_name()

        id_list = [int(id_str) for id_str in
                        cloud_conf.get_value_list(GET_ANY_ID_XPATH)]
        target_id = ID_MIN
        while True:
            if target_id in id_list:
                target_id += 1
                continue
            break

        try:
            targetlist = cloud_conf.get_node(TO_LIST_TAG_NANE[target_name])
        except NotFoundError:
            cloud_conf.append_child(TO_LIST_TAG_NANE[target_name])
            targetlist = cloud_conf.get_node(TO_LIST_TAG_NANE[target_name])

        added_node.append_child(ID_NAME, str(target_id))
        targetlist.append_child(added_node)

        pal_entry = Entry(ReqTypes.ADD, CLOUD_NAME)
        pal_entry.get().append_child(added_node.get_clone())

        self.__try_append_proxy_tag(pal_entry.get())

        try:
            result_entry = self.send_entry(pal_entry, pal_info.PAL_CLOUD_PATH)
            if result_entry.get_result() != Results.OK:
                self.error("Add error")
                entry.set_result(result_entry.get_result())
                return entry
        except:
            self.exception("Add error")
            entry.set_result(Results.ADD_ERROR)
            return entry

        servicectrl_suspend([], [NASDSYNC2_SERVICE])
        self.__save_config(CLOUD_CONF_FILE, cloud_conf)
        servicectrl_resume([], [NASDSYNC2_SERVICE])
        entry.set_result(Results.OK)
        return entry

    def __modify_handler(self, entry):
        """Handle modify request and set entry to result."""

        modified_node = entry.get().get_node(ANY_CHILD_NAME)
        cloud_conf = self.__load_config(CLOUD_CONF_FILE)
        target_node = cloud_conf.get_node(FIND_BY_ID_XPATH_TMPL %
                            (modified_node.get_node(ID_NAME).get_text(),))
        old_name = target_node.get_value(NAME_NAME)
        target_node.update(modified_node)

        pal_entry = Entry(ReqTypes.MODIFY, CLOUD_NAME)
        pal_new_target_node = target_node.get_clone()
        if pal_new_target_node.get_value(NAME_NAME) != old_name:
            pal_new_target_node.append_child(
                            XmlConfNode(OLDNAME_XML_TMP % (old_name)))
        pal_entry.get().append_child(pal_new_target_node)

        self.__try_append_proxy_tag(pal_entry.get())

        try:
            result_entry = self.send_entry(pal_entry, pal_info.PAL_CLOUD_PATH)
            if result_entry.get_result() != Results.OK:
                self.error("Modify error")
                entry.set_result(result_entry.get_result())
                return entry
        except:
            self.exception("Modify error")
            entry.set_result(Results.MODIFY_ERROR)
            return entry

        servicectrl_suspend([], [NASDSYNC2_SERVICE])
        self.__save_config(CLOUD_CONF_FILE, cloud_conf)
        servicectrl_resume([], [NASDSYNC2_SERVICE])
        entry.set_result(Results.OK)
        return entry

    def __delete_handler(self, entry):
        """Handle delete request and set entry to result."""

        deleted_node = entry.get().get_node(ANY_CHILD_NAME)
        cloud_conf = self.__load_config(CLOUD_CONF_FILE)

        for list_node in cloud_conf.get_node_list(ANY_CHILD_NAME):
            idx = 0
            for item in list_node.get_node_list(ANY_CHILD_NAME):
                if (item.get_value(ID_NAME) !=
                        deleted_node.get_value(ID_NAME)):

                    idx += 1
                    continue

                pal_entry = Entry(ReqTypes.DELETE, CLOUD_NAME)
                pal_entry.get().append_child(item.get_clone())

                self.__try_append_proxy_tag(pal_entry.get())

                try:
                    result_entry = \
                            self.send_entry(pal_entry, pal_info.PAL_CLOUD_PATH)
                    if result_entry.get_result() != Results.OK:
                        self.error("Delete error")
                        entry.set_result(result_entry.get_result())
                        return entry
                except:
                    self.exception("Delete error")
                    entry.set_result(Results.DELETE_ERROR)
                    return entry

                list_node.delete_child(ANY_CHILD_NAME, index=idx)

        servicectrl_suspend([], [NASDSYNC2_SERVICE])
        self.__save_config(CLOUD_CONF_FILE, cloud_conf)
        servicectrl_resume([], [NASDSYNC2_SERVICE])
        entry.set_result(Results.OK)
        return entry

    def __init__(self, option=None):
        """Constructor."""

        falcmd.FalCmd.__init__(self, option=option)

        save_and_load_handler = lambda entry: self.send_entry_with_file(
            entry, pal_info.PAL_CONFFILE_PATH, CLOUD_CONF_FILE)

        pass_handler = \
            lambda entry: self.send_entry(entry, pal_info.PAL_CLOUD_PATH)

        self.set_handler(ReqTypes.SAVE, save_and_load_handler)
        self.set_handler(ReqTypes.LOAD, save_and_load_handler)
        self.set_handler(ReqTypes.GET, pass_handler)
        self.set_handler(ReqTypes.CHECK, self.__check_handler)
        self.set_handler(ReqTypes.ADD, self.__add_handler)
        self.set_handler(ReqTypes.MODIFY, self.__modify_handler)
        self.set_handler(ReqTypes.DELETE, self.__delete_handler)

#
# Main
#

if __name__ == "__main__":
    FalCloud(option=falcmd.FalCmdOption()).handle_request()
